==> fred-backend-identity_1.0.0.orig.tar.gz: fred-backend-identity-1.0.0/src/impl/lib/create_contact_state_flag_request.cc <==
/*
 * Copyright (C) 2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/create_contact_state_flag_request.hh"
#include "src/impl/lib/unwrap_uuid_from_psql.hh"
#include "src/impl/lib/wrap_uuid_into_psql.hh"
#include "src/impl/exceptions.hh"

#include "liblog/liblog.hh"

#include "libpg/query.hh"
#include "libpg/sql_state/code.hh"

#include <boost/uuid/uuid_io.hpp>

#include <spdlog/fmt/fmt.h>

#include <sstream>
#include <string>
#include <tuple>
#include <utility>


namespace Fred {
namespace Identity {
namespace Impl {
namespace Lib {

std::string wrap_into_psql_representation(ContactStateFlag::ServerContactNameChangeProhibited)
{
    return "serverContactNameChangeProhibited";
}

std::string wrap_into_psql_representation(ContactStateFlag::ServerContactOrganizationChangeProhibited)
{
    return "serverContactOrganizationChangeProhibited";
}

std::string wrap_into_psql_representation(ContactStateFlag::ServerContactIdentChangeProhibited)
{
    return "serverContactIdentChangeProhibited";
}

std::string wrap_into_psql_representation(ContactStateFlag::ServerContactPermanentAddressChangeProhibited)
{
    return "serverContactPermanentAddressChangeProhibited";
}

template <typename T>
void create_contact_state_flag_request(
        const LibPg::PgRwTransaction& tx,
        const ContactUuid& contact_uuid,
        ContactStateFlag::Type<T> contact_state_flag)
{
    try
    {
        using RequestId = StrongType<unsigned long long, struct RequestIdTag_>;
        const auto dbres = exec(
                tx,
                LibPg::make_query(
                        "INSERT INTO object_state_request (object_id, state_id) "
                        "SELECT obr.id, eos.id "
                        "FROM object_registry obr "
                        "LEFT JOIN enum_object_states eos ON eos.name = ") << LibPg::parameter<ContactStateFlag::Type<T>>().as_varchar() << " AND "
                                                            "obr.type = ANY(eos.types) AND "
                                                            "eos.manual "
                        "WHERE obr.uuid = " << LibPg::parameter<ContactUuid>().as_type("UUID") << " AND "
                              "obr.erdate IS NULL AND "
                              "obr.type = get_object_type_id('contact') "
                        "RETURNING" << LibPg::item<RequestId>() << "id",
                contact_state_flag,
                contact_uuid);
        if (dbres.empty())
        {
            struct ContactNotFound : ContactDoesNotExist
            {
                const char* what() const noexcept override { return "no contact by uuid and history_uuid found"; }
            };
            throw ContactNotFound{};
        }
        if (LibPg::RowIndex{1} < dbres.size())
        {
            struct UnexpectedDatabaseResult : Exception
            {
                const char* what() const noexcept override { return "too many contacts was found"; }
            };
            throw UnexpectedDatabaseResult{};
        }
    }
    catch (const LibPg::ExecFailure& e)
    {
        if (e.error_code() == LibPg::SqlState::not_null_violation)
        {
            struct ContactStateFlagNotAvailable : Exception
            {
                const char* what() const noexcept override
                {
                    static const auto msg =
                            "contact state flag " + wrap_into_psql_representation(ContactStateFlag::Type<T>{}) + " "
                            "not available";
                    return msg.c_str();
                }
            };
            throw ContactStateFlagNotAvailable{};
        }
        throw;
    }
}

template void create_contact_state_flag_request(const LibPg::PgRwTransaction&, const ContactUuid&, ContactStateFlag::ServerContactIdentChangeProhibited);
template void create_contact_state_flag_request(const LibPg::PgRwTransaction&, const ContactUuid&, ContactStateFlag::ServerContactNameChangeProhibited);
template void create_contact_state_flag_request(const LibPg::PgRwTransaction&, const ContactUuid&, ContactStateFlag::ServerContactOrganizationChangeProhibited);
template void create_contact_state_flag_request(const LibPg::PgRwTransaction&, const ContactUuid&, ContactStateFlag::ServerContactPermanentAddressChangeProhibited);

}//namespace Fred::Identity::Impl::Lib
}//namespace Fred::Identity::Impl
}//namespace Fred::Identity
}//namespace Fred

==> fred-backend-identity_1.0.1.orig.tar.gz: fred-backend-identity-1.0.1/src/impl/lib/create_contact_state_flag_request.cc <==
/*
 * Copyright (C) 2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/create_contact_state_flag_request.hh"
#include "src/impl/lib/unwrap_uuid_from_psql.hh"
#include "src/impl/lib/wrap_uuid_into_psql.hh"
#include "src/impl/exceptions.hh"

#include "liblog/liblog.hh"

#include "libpg/query.hh"
#include "libpg/sql_state/code.hh"

#include <boost/uuid/uuid_io.hpp>

#include <spdlog/fmt/fmt.h>

#include <sstream>
#include <string>
#include <tuple>
#include <utility>


namespace Fred {
namespace Identity {
namespace Impl {
namespace Lib {

std::string wrap_into_psql_representation(ContactStateFlag::ServerContactNameChangeProhibited)
{
    return "serverContactNameChangeProhibited";
}

std::string wrap_into_psql_representation(ContactStateFlag::ServerContactOrganizationChangeProhibited)
{
    return "serverContactOrganizationChangeProhibited";
}

std::string wrap_into_psql_representation(ContactStateFlag::ServerContactIdentChangeProhibited)
{
    return "serverContactIdentChangeProhibited";
}

std::string wrap_into_psql_representation(ContactStateFlag::ServerContactPermanentAddressChangeProhibited)
{
    return "serverContactPermanentAddressChangeProhibited";
}

template <typename T>
void create_contact_state_flag_request(
        const LibPg::PgRwTransaction& tx,
        const ContactUuid& contact_uuid,
        ContactStateFlag::Type<T> contact_state_flag)
{
    try
    {
        using ObjectId = StrongType<unsigned long long, struct ObjectIdTag_>;
        const auto dbres = exec(
                tx,
                LibPg::make_query(
                        "INSERT INTO object_state_request (object_id, state_id) "
                        "SELECT obr.id, eos.id "
                        "FROM object_registry obr "
                        "LEFT JOIN enum_object_states eos ON eos.name = ") << LibPg::parameter<ContactStateFlag::Type<T>>().as_varchar() << " AND "
                                                            "obr.type = ANY(eos.types) AND "
                                                            "eos.manual "
                        "WHERE obr.uuid = " << LibPg::parameter<ContactUuid>().as_type("UUID") << " AND "
                              "obr.erdate IS NULL AND "
                              "obr.type = get_object_type_id('contact') "
                        "RETURNING" << LibPg::item<ObjectId>() << "object_id",
                contact_state_flag,
                contact_uuid);
        if (dbres.empty())
        {
            struct ContactNotFound : ContactDoesNotExist
            {
                const char* what() const noexcept override { return "no contact by uuid and history_uuid found"; }
            };
            throw ContactNotFound{};
        }
        if (LibPg::RowIndex{1} < dbres.size())
        {
            struct UnexpectedDatabaseResult : Exception
            {
                const char* what() const noexcept override { return "too many contacts was found"; }
            };
            throw UnexpectedDatabaseResult{};
        }
        const auto object_id = dbres.front().template get<ObjectId>();
        exec(
                tx,
                LibPg::make_query(
                        "SELECT update_object_states(") << LibPg::parameter<ObjectId>().as_big_int() << ")",
                object_id);
    }
    catch (const LibPg::ExecFailure& e)
    {
        if (e.error_code() == LibPg::SqlState::not_null_violation)
        {
            struct ContactStateFlagNotAvailable : Exception
            {
                const char* what() const noexcept override
                {
                    static const auto msg =
                            "contact state flag " + wrap_into_psql_representation(ContactStateFlag::Type<T>{}) + " "
                            "not available";
                    return msg.c_str();
                }
            };
            throw ContactStateFlagNotAvailable{};
        }
        throw;
    }
}

template void create_contact_state_flag_request(const LibPg::PgRwTransaction&, const ContactUuid&, ContactStateFlag::ServerContactIdentChangeProhibited);
template void create_contact_state_flag_request(const LibPg::PgRwTransaction&, const ContactUuid&, ContactStateFlag::ServerContactNameChangeProhibited);
template void create_contact_state_flag_request(const LibPg::PgRwTransaction&, const ContactUuid&, ContactStateFlag::ServerContactOrganizationChangeProhibited);
template void create_contact_state_flag_request(const LibPg::PgRwTransaction&, const ContactUuid&, ContactStateFlag::ServerContactPermanentAddressChangeProhibited);

}//namespace Fred::Identity::Impl::Lib
}//namespace Fred::Identity::Impl
}//namespace Fred::Identity
}//namespace Fred
